#include "pch.hpp"
#include "chalcopyrite.hpp"
#include "mainwnd.hpp"
#include "scorelinedlg.hpp"
#include "qibfilterdlg.hpp"
#include "rankprofdlg.hpp"
#include "trackrecorddlg.hpp"
#include "scoreline.hpp"
#include "shortcut.hpp"
#include "characters.hpp"
#include "TextFileWriter.hpp"
#include "Formatter.hpp"
#include "memento.hpp"
#include "SwrAddrDef.h"
#include "resource.h"

#define MINIMAL_USE_PROCESSHEAPARRAY
#include "MinimalArray.hpp"

typedef Minimal::ProcessHeapArrayT<SCORELINE_FILTER> ScoreLineFilterArray;

#define UC_VIEWALL  0xDEA0
#define UC_VIEWL30  0xDEA1
#define UC_VIEWL50  0xDEA2
#define UC_VIEWL100 0xDEA3
#define UC_VIEWTOD  0xDEA4
#define UC_VIEWYES  0xDEA5
#define UC_VIEWCST  0xDEA6
#define UC_VIEWMAX  0xDEA7

#define UC_DIRP1	0xDCE0
#define UC_DIRP2	0xDCE1

#define UC_OUTHTML  0xDEE2
#define UC_OPENHIT  0xDEE3

#define UC_UNDOSCR 0xBEEF
#define UC_REDOSCR 0xBEAF

#define UC_RNKPROF 0xFEED
#define UC_TRKRECD 0xFEEE

// GfBbg_uNtbNp
#define PROP_OPROC	"hkskEditProc"


enum STD_VIEWMODE {
	STD_VIEW_ALL,
	STD_VIEW_L30,
	STD_VIEW_L50,
	STD_VIEW_L100,
	STD_VIEW_TODAY,
	STD_VIEW_YESTERDAY,
	STD_VIEW_CUSTOM,
};

static BYTE const s_convTable[] = {
	 0, 2, 5, 1,
	 3, 6, 7, 4,
	 8,10,11, 9,
	12,13,14,
};

static Minimal::ProcessHeapStringT<char> s_label_specific[SWRCHAR_MAX + 1];
static Minimal::ProcessHeapStringT<char> s_label_overall[SWRCHAR_MAX + 1];

static HMENU s_hSysMenu;
static BOOL  s_trDirLeft;	/* ScoreLineW[ɑgݍ񂾕ǂH */
static int   s_tabIndex;
static int   s_viewMode;
static ScoreLineFilterArray s_custFilters;

// ^uhbOtbNp
static WNDPROC s_origTabProc;
static HCURSOR s_origCursor;
static int  s_dragTabIndex;
static BOOL s_downBeforeMove;

static bool ScoreLineDialog_Reflesh(HWND hDlg, int tabIndex);


void OutputToHTML(LPCSTR fileName)
{
	int sum[2];
	int sumAll[2];
	int winRate;
	int n, win, lose;

	TextFileWriter writer;
	if(FAILED(writer.Initialize(fileName)))
		return;

	writer.WriteFormatted(
		"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" "
			"\"http://www.w3.org/TR/html4/loose.dtd\">"
		"<HTML><HEAD>"
			"<TITLE></TITLE>"
			"<META http-equiv=\"content-type\" content=\"text/html; charset=Shift_JIS\">"
			"<STYLE type=\"text/css\">"
			"TABLE{width:%dem;empty-cells:hide;}"
			"TH{width:6em;}"
			"TD{text-align:center;}"
			"</STYLE>"
		"</HEAD>"
		"<BODY>"
		"<TABLE border=\"1\"><TR><TH></TH>",
		(ARRAYSIZE(g_characters) + 1) * 6);

	for(int i = 0; i < ARRAYSIZE(g_characters); ++i)
		writer.WriteFormatted("<TH colspan=\"2\">%s</TH>", g_characters[i].abbr);
	writer.Write("<TH COLSPAN=\"2\">v</TH></TR>");

	sumAll[0] = sumAll[1] = 0;
	for(int i = 0; i < ARRAYSIZE(g_characters); ++i) {
		writer.WriteFormatted("<TR><TH rowspan=\"2\">%s</TH>", g_characters[i].abbr);
		sum[0] = sum[1] = 0;
		for(int j = 0; j < ARRAYSIZE(g_characters); ++j) {
			for(int k = 0; k < 2; ++k) {
				n = ScoreLine_Read(i, j, k);
				sum[k] += n;
				writer.WriteFormatted("<TD>%d</TD>", n);
			}
		}
		for(int k = 0; k < 2; ++k)
			writer.WriteFormatted("<TD>%d</TD>", sum[k]);

		writer.Write("</TR><TR>");
		for(int j = 0; j < ARRAYSIZE(g_characters); ++j) {
			win = ScoreLine_Read(i, j, 0); lose = ScoreLine_Read(i, j, 1);
			if(win + lose == 0) {
				writer.Write("<TD colspan=\"2\">--.--%</TD>");
			} else {
				winRate = MulDiv(win, 10000, win + lose);
				writer.WriteFormatted("<TD colspan=\"2\">%d.%02d%%</TD>", winRate / 100, winRate % 100);
			}
		}
		if(sum[0] + sum[1] == 0) {
			writer.Write("<TD colspan=\"2\">--.--%</TD></TR>");
		} else {
			winRate = MulDiv(sum[0], 10000, sum[0] + sum[1]);
			writer.WriteFormatted("<TD colspan=\"2\">%d.%02d%%</TD></TR>", winRate / 100, winRate % 100);
		}
		sumAll[0] += sum[0]; sumAll[1] += sum[1];
	}
	writer.WriteFormatted(
		"<TR><TD colspan=\"%d\"></TD><TH colspan=\"2\" rowspan=\"2\">v</TH>",
		ARRAYSIZE(g_characters) * 2 - 1);
	writer.WriteFormatted(
		"<TD>%d</TD><TD>%d</TD></TR>",
		sumAll[0], sumAll[1]);
	writer.WriteFormatted(
		"<TR><TD colspan=\"%d\"></TD>",
		ARRAYSIZE(g_characters) * 2 + 1);
	if(sumAll[0] + sumAll[1] == 0) {
		writer.Write("<TD colspan=\"2\">--.--%</TD></TR>");
	} else {
		winRate = ::MulDiv(sumAll[0], 10000, sumAll[0] + sumAll[1]);
		writer.WriteFormatted("<TD colspan=\"2\">%d.%02d%%</TD></TR>", winRate / 100, winRate % 100);
	}
	writer.Write("</TABLE></BODY></HTML>");
}


LRESULT TabCtrl_OnLButtonDown(HWND hWnd, BOOL F, int x, int y, WPARAM wParam)
{
	TC_HITTESTINFO tchi = {{x, y}, TCHT_ONITEM};
	int i = TabCtrl_HitTest(hWnd, &tchi);
	if(i != -1) {
		// fɑI
		TabCtrl_SetCurSel(hWnd, i);
		NMHDR nmhdr = { hWnd, IDC_TAB, TCN_SELCHANGE };
		::SendMessage(GetParent(hWnd), WM_NOTIFY, IDC_TAB, (LPARAM)&nmhdr);
		return TRUE;
	} else {
		return ::CallWindowProc(s_origTabProc, hWnd, WM_LBUTTONDOWN, wParam, MAKELONG(x, y));
	}
}

LRESULT TabCtrl_OnMouseMove(HWND hWnd, int x, int y, UINT keyFlags)
{
	if(keyFlags == MK_LBUTTON) {	// hbOmF
		if(!s_origCursor) { // t@[XgCpNg
			TC_HITTESTINFO tchi = {{x, y}, TCHT_ONITEM};
			int i = TabCtrl_HitTest(hWnd, &tchi);
			if(i != -1) {
				s_origCursor = SetCursor(LoadCursor(NULL, IDC_HAND));
				s_dragTabIndex = i;
				TabCtrl_SetCurSel(hWnd, i);
				NMHDR nmhdr = { hWnd, IDC_TAB, TCN_SELCHANGE };
				::SendMessage(GetParent(hWnd), WM_NOTIFY, IDC_TAB, (LPARAM)&nmhdr);

				::SetCapture(hWnd);
			}
		} else {			// ZJhCpNg
			TC_HITTESTINFO tchi = {{x, y}, TCHT_ONITEM};
			int i = TabCtrl_HitTest(hWnd, &tchi);
			if(i != -1) {	// ͈͓hbO
				::SetCursor(LoadCursor(NULL, IDC_HAND));
				if(i != s_dragTabIndex) {
					TC_ITEM item;
					char itemText[64];
					item.mask = TCIF_TEXT | TCIF_PARAM;
					item.pszText = itemText;
					item.cchTextMax = sizeof itemText;
					TabCtrl_GetItem(hWnd, s_dragTabIndex, &item);
					TabCtrl_DeleteItem(hWnd, s_dragTabIndex);
					TabCtrl_InsertItem(hWnd, i, &item);
					s_dragTabIndex = i;
				}
			} else {		// ͈͊OhbO
				// ́A__I`_I
				::SetCursor(LoadCursor(NULL, IDC_NO));
			}
		}
	}
	return TRUE;
}

LRESULT TabCtrl_OnLButtonUp(HWND hWnd, int x, int y, WPARAM wParam)
{
	if(s_origCursor) {
		/* J[\AACRɖ߂ */
		::ReleaseCapture();
		::SetCursor(s_origCursor);
		s_origCursor = NULL;
	}

	return ::CallWindowProc(s_origTabProc, hWnd, WM_LBUTTONUP, wParam, MAKELONG(x, y));
}


static LRESULT CALLBACK TabCtrl_WindowProc(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch(Msg) {
	HANDLE_MSG(hWnd, WM_LBUTTONDOWN, TabCtrl_OnLButtonDown);
	HANDLE_MSG(hWnd, WM_MOUSEMOVE, TabCtrl_OnMouseMove);
	HANDLE_MSG(hWnd, WM_LBUTTONUP, TabCtrl_OnLButtonUp);
	}
	return ::CallWindowProc(s_origTabProc, hWnd, Msg, wParam, lParam);
}

static void EditCtrl_OnButtonDoubleClick_Common(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags, int score)
{
	if(s_tabIndex == 0)
		return;

	int nID = GetDlgCtrlID(hwnd);
	int index = (nID - IDC_EDIT110);

	SCORELINE_ITEM item;

	SYSTEMTIME loctime;
	::GetLocalTime(&loctime);
	::SystemTimeToFileTime(&loctime, (LPFILETIME)&item.timestamp);
	if(s_trDirLeft) {
		item.p1name[0] = 0;
		item.p1id  = s_tabIndex - 1;
		item.p1win = (index & 1) ? 0: score;
		item.p2name[0] = 0;
		item.p2id  = s_convTable[index >> 1];
		item.p2win = (index & 1) ? score : 0;
	} else {
		item.p1name[0] = 0;
		item.p1id  = s_convTable[index >> 1];
		item.p1win = (index & 1) ? score : 0;
		item.p2name[0] = 0;
		item.p2id  = s_tabIndex - 1;
		item.p2win = (index & 1) ? 0: score;
	}

	if(ScoreLine_Append(&item)) {
		Memento_Append(&item);
		ScoreLineDialog_Reflesh(GetParent(hwnd), s_tabIndex);
	}
}

static void EditCtrl_OnLButtonDoubleClick(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
{
	EditCtrl_OnButtonDoubleClick_Common(hwnd, fDoubleClick, x, y, keyFlags, 2);
}

static void EditCtrl_OnRButtonDoubleClick(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
{
	EditCtrl_OnButtonDoubleClick_Common(hwnd, fDoubleClick, x, y, keyFlags, -2);
}

static LRESULT CALLBACK EditCtrl_WindowProc(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch(Msg) {
	HANDLE_MSG(hWnd, WM_RBUTTONDBLCLK, EditCtrl_OnRButtonDoubleClick);
	HANDLE_MSG(hWnd, WM_LBUTTONDBLCLK, EditCtrl_OnLButtonDoubleClick);
	case WM_RBUTTONDOWN: return TRUE;
	case WM_RBUTTONUP:   return TRUE;
	}
	WNDPROC origProc = (WNDPROC)GetProp(hWnd, PROP_OPROC);
	return ::CallWindowProc(origProc, hWnd, Msg, wParam, lParam);
}

static void ScoreLine_AppendViewFilter(ScoreLineFilterArray &ret)
{
	SYSTEMTIME loctime, begin, end;
	SCORELINE_FILTER flt;
	switch(s_viewMode) {
	case STD_VIEW_TODAY:
		// n_
		GetLocalTime(&loctime);
		begin = loctime;
		begin.wHour =
		begin.wMinute = 
		begin.wSecond =
		begin.wMilliseconds = 0;
		flt.type = SCORELINE_FILTER_TIMESTAMP_BEGIN;
		::SystemTimeToFileTime(&begin, (LPFILETIME)&flt.timestamp);
		ret.Push(flt);
		// I_
		end = loctime;
		end.wHour = 23;
		end.wMinute = 59;
		end.wSecond = 59;
		end.wMilliseconds = 999;
		flt.type = SCORELINE_FILTER_TIMESTAMP_END;
		::SystemTimeToFileTime(&end, (LPFILETIME)&flt.timestamp);
		ret.Push(flt);
		break;
	case STD_VIEW_YESTERDAY:
		// n_
		GetLocalTime(&loctime);
		begin = loctime;
		begin.wHour =
		begin.wMinute = 
		begin.wSecond =
		begin.wMilliseconds = 0;
		flt.type = SCORELINE_FILTER_TIMESTAMP_BEGIN;
		::SystemTimeToFileTime(&begin, (LPFILETIME)&flt.timestamp);
		/* 24 x 60 x 60b x 1000~b x 1000}CNb x 10imb */
		flt.timestamp -= 864000000000ULL; 
		ret.Push(flt);
		// I_
		end = loctime;
		end.wHour = 23;
		end.wMinute = 59;
		end.wSecond = 59;
		end.wMilliseconds = 999;
		flt.type = SCORELINE_FILTER_TIMESTAMP_END;
		::SystemTimeToFileTime(&end, (LPFILETIME)&flt.timestamp);
		flt.timestamp -= 864000000000ULL;
		ret.Push(flt);
		break;
	case STD_VIEW_CUSTOM:
		ret += s_custFilters;
		break;
	}
}

static void ScoreLineDialog_RefleshUnit(HWND hDlg, int index, int sumWin, int sumLose, BOOL edittable)
{
	int winRate;

	::SetDlgItemInt(hDlg, IDC_EDIT110 + index * 2, sumWin, FALSE);
	::SetDlgItemInt(hDlg, IDC_EDIT111 + index * 2, sumLose, FALSE);
	::SendDlgItemMessage(hDlg, IDC_EDIT110 + index * 2, EM_SETREADONLY, !edittable, 0);
	::SendDlgItemMessage(hDlg, IDC_EDIT111 + index * 2, EM_SETREADONLY, !edittable, 0);
	if(sumWin + sumLose == 0) {
		::SetDlgItemText(hDlg, IDC_LABELP11 + index, "--.--%");
	} else {
		winRate = ::MulDiv(sumWin, 10000, sumWin + sumLose);
		;
		::SetDlgItemText(hDlg, IDC_LABELP11 + index, 
			Formatter("%d.%02d%%", winRate / 100, winRate % 100));
	}
}

static void ScoreLineDialog_RefleshLeft(HWND hDlg, int tabIndex)
{
	int i, j;
	int sumWin, sumLose;
	int sumWinAll, sumLoseAll;

	if(tabIndex == 0) {
		for(i = 0; i < ARRAYSIZE(s_label_overall); ++i)
			::SetDlgItemText(hDlg, IDC_LABEL11 + i, s_label_overall[i]);
		sumWinAll = sumLoseAll = 0;
		for(i = 0; i < SWRCHAR_MAX; ++i) {
			sumWin = sumLose = 0;
			for(j = 0; j < SWRCHAR_MAX; ++j) {
				sumWin  += ScoreLine_Read(s_convTable[i], j, 0);
				sumLose += ScoreLine_Read(s_convTable[i], j, 1);
			}
			sumWinAll  += sumWin;
			sumLoseAll += sumLose;

			ScoreLineDialog_RefleshUnit(hDlg, i, sumWin, sumLose, FALSE);
		}
	} else {
		for(i = 0; i < ARRAYSIZE(s_label_specific); ++i)
			::SetDlgItemText(hDlg, IDC_LABEL11 + i, s_label_specific[i]);
		sumWinAll = sumLoseAll = 0;
		for(i = 0; i < SWRCHAR_MAX; ++i) {
			sumWin  = ScoreLine_Read(tabIndex - 1, s_convTable[i], 0);
			sumLose = ScoreLine_Read(tabIndex - 1, s_convTable[i], 1);
			sumWinAll  += sumWin;
			sumLoseAll += sumLose;

			ScoreLineDialog_RefleshUnit(hDlg, i, sumWin, sumLose, FALSE);
		}
	}
	ScoreLineDialog_RefleshUnit(hDlg, 15, sumWinAll, sumLoseAll, FALSE);
}

static void ScoreLineDialog_RefleshRight(HWND hDlg, int tabIndex)
{
	int i, j;
	int sumWin, sumLose;
	int sumWinAll, sumLoseAll;

	if(tabIndex == 0) {
		for(i = 0; i < ARRAYSIZE(s_label_specific); ++i)
			::SetDlgItemText(hDlg, IDC_LABEL11 + i, s_label_specific[i]);
		sumWinAll = sumLoseAll = 0;
		for(i = 0; i < SWRCHAR_MAX; ++i) {
			sumWin = sumLose = 0;
			for(j = 0; j < SWRCHAR_MAX; ++j) {
				sumWin  += ScoreLine_Read(j, s_convTable[i], 0);
				sumLose += ScoreLine_Read(j, s_convTable[i], 1);
			}
			sumWinAll  += sumWin;
			sumLoseAll += sumLose;

			ScoreLineDialog_RefleshUnit(hDlg, i, sumWin, sumLose, FALSE);
		}
	} else {
		for(i = 0; i < ARRAYSIZE(s_label_overall); ++i)
			::SetDlgItemText(hDlg, IDC_LABEL11 + i, s_label_overall[i]);
		sumWinAll = sumLoseAll = 0;
		for(i = 0; i < SWRCHAR_MAX; ++i) {
			sumWin  = ScoreLine_Read(s_convTable[i], tabIndex - 1, 0);
			sumLose = ScoreLine_Read(s_convTable[i], tabIndex - 1, 1);
			sumWinAll  += sumWin;
			sumLoseAll += sumLose;

			ScoreLineDialog_RefleshUnit(hDlg, i, sumWin, sumLose, FALSE);
		}
	}
	ScoreLineDialog_RefleshUnit(hDlg, 15, sumWinAll, sumLoseAll, FALSE);
}

static bool ScoreLineDialog_QueryTrackRecord(int limit)
{
	ScoreLine_Enter();

	int ret = (limit == -1) ?
		ScoreLine_QueryTrackRecord(NULL, 0)
	:	ScoreLine_QueryTrackRecord(NULL, 0, limit);
	if(!ret) {
		ScoreLine_Leave(true);
		return false;
	}

	ScoreLine_Leave(false);
	return true;
}

static bool ScoreLineDialog_QueryTrackRecord(ScoreLineFilterArray &flt)
{
	ScoreLine_Enter();

	if(!ScoreLine_QueryTrackRecord(flt.GetRaw(), flt.GetSize())) {
		ScoreLine_Leave(true);
		return false;
	}

	ScoreLine_Leave(false);
	return true;
}

static bool ScoreLineDialog_Reflesh(HWND hDlg, int tabIndex)
{
	switch(s_viewMode) {
	case STD_VIEW_ALL:
		if(!ScoreLineDialog_QueryTrackRecord(-1))
			return false;
		break;
	case STD_VIEW_L30:
		if(!ScoreLineDialog_QueryTrackRecord(30))
			return false;
		break;
	case STD_VIEW_L50:
		if(!ScoreLineDialog_QueryTrackRecord(50))
			return false;
		break;
	case STD_VIEW_L100:
		if(!ScoreLineDialog_QueryTrackRecord(100))
			return false;
		break;
	case STD_VIEW_TODAY:
	case STD_VIEW_YESTERDAY:
	case STD_VIEW_CUSTOM:
		{
			ScoreLineFilterArray flt;
			ScoreLine_AppendViewFilter(flt);
			if(!ScoreLineDialog_QueryTrackRecord(flt))
				return false;
		}
	}

	Minimal::ProcessHeapStringT<char> baseName;
	baseName = ScoreLine_GetPath();
	::PathStripPath(baseName);
	::PathRemoveExtension(baseName);
	baseName.Repair();
	::SetWindowText(hDlg,
		Formatter(WINDOW_TEXT " - %s",
			static_cast<LPCSTR>(baseName)));

	if(s_trDirLeft) {
		ScoreLineDialog_RefleshLeft(hDlg, tabIndex);
	} else {
		ScoreLineDialog_RefleshRight(hDlg, tabIndex);
	}
	return true;
}

static void ScoreLineDialog_OpenProfile(HWND hDlg, LPCSTR fileName)
{
	Minimal::ProcessHeapStringT<char> oldPath(ScoreLine_GetPath());
	ScoreLine_SetPath(fileName);
	if(ScoreLine_Open(true)) {
		Memento_Reset();
		ScoreLineDialog_Reflesh(hDlg, s_tabIndex);
	} else {
		ScoreLine_SetPath(oldPath);
		::MessageBox(hDlg, "vt@C̃}bsOɎs܂B", NULL, MB_OK | MB_ICONSTOP);
	}
}

static void SysMenu_OnClose(HWND hDlg, int x, int y)
{
	::ShowWindow(hDlg, SW_HIDE);
}

static void SysMenu_OnView(HWND hDlg, int nID, int x, int y)
{
	int newMode = nID - UC_VIEWALL;
	if(newMode != s_viewMode) {
		::CheckMenuItem(s_hSysMenu, nID, MF_CHECKED);
		for(int i = UC_VIEWALL; i < UC_VIEWMAX; ++i)
			if(i != nID) ::CheckMenuItem(s_hSysMenu, i, MF_UNCHECKED);
		s_viewMode = newMode;
		ScoreLineDialog_Reflesh(hDlg, s_tabIndex);
	}
}

static void SysMenu_OnViewCustom(HWND hDlg, int nID, int x, int y)
{
	SCORELINE_FILTER flt;

	if(
	::DialogBoxA(
	  g_hInstance,
	  MAKEINTRESOURCE(IDD_QIBFILTER),
	  hDlg,
	  QIBFilterDlg_DlgProc) == IDOK) {
		s_custFilters.Clear();
		if(g_fltP2NameEnabled) {
			flt.type = SCORELINE_FILTER_P2NAME;
			::lstrcpy(flt.name, g_fltP2Name);
			s_custFilters.Push(flt);
		}
		if(g_fltDateBeginEnabled) {
			flt.type = SCORELINE_FILTER_TIMESTAMP_BEGIN;
			flt.timestamp = g_fltDateBegin;
			s_custFilters.Push(flt);
		}
		if(g_fltDateEndEnabled) {
			flt.type = SCORELINE_FILTER_TIMESTAMP_END;
			flt.timestamp = g_fltDateEnd;
			s_custFilters.Push(flt);
		}
		::CheckMenuItem(s_hSysMenu, nID, MF_CHECKED);
		for(int i = UC_VIEWALL; i < UC_VIEWMAX; ++i)
			if(i != nID) ::CheckMenuItem(s_hSysMenu, i, MF_UNCHECKED);
		s_viewMode = STD_VIEW_CUSTOM;
		ScoreLineDialog_Reflesh(hDlg, s_tabIndex);
	}
}

static void SysMenu_OnOutputToHTML(HWND hDlg, int x, int y)
{
	char fileName[1025];
	OPENFILENAME ofn;
	::ZeroMemory(&ofn, sizeof ofn);
	ofn.lStructSize = sizeof ofn;
	ofn.hwndOwner = hDlg;
	ofn.lpstrFile = fileName;
	ofn.nMaxFile = sizeof fileName;
	ofn.lpstrDefExt = "html";
	ofn.lpstrFilter = "HTMLt@C (*.html)\0*.html\0";
	ofn.Flags = OFN_OVERWRITEPROMPT | OFN_NOCHANGEDIR;
	fileName[0] = 0;
	if(::GetSaveFileName(&ofn)) {
		OutputToHTML(fileName);
	}
}

static void SysMenu_OnOpenProfile(HWND hDlg, int x, int y)
{
	char fileName[1025];
	OPENFILENAME ofn;
	::ZeroMemory(&ofn, sizeof ofn);
	ofn.lStructSize = sizeof ofn;
	ofn.hwndOwner = hDlg;
	ofn.lpstrFile = fileName;
	ofn.nMaxFile = sizeof fileName;
	ofn.lpstrDefExt = "db";
	ofn.lpstrFilter = "Trackrecord Database (*.db)\0*.db\0";
	ofn.Flags = OFN_CREATEPROMPT | OFN_NOCHANGEDIR;
	fileName[0] = 0;
	if(::GetOpenFileName(&ofn)) {
		ScoreLineDialog_OpenProfile(hDlg, fileName);
	}
}

static void SysMenu_OnDirectionP1(HWND hDlg, int x, int y)
{
	if(!s_trDirLeft) {
		::CheckMenuItem(s_hSysMenu, UC_DIRP1, MF_CHECKED);
		::CheckMenuItem(s_hSysMenu, UC_DIRP2, MF_UNCHECKED);
		s_trDirLeft = TRUE;
		ScoreLineDialog_Reflesh(hDlg, s_tabIndex);
	}
}

static void SysMenu_OnDirectionP2(HWND hDlg, int x, int y)
{
	if(s_trDirLeft) {
		::CheckMenuItem(s_hSysMenu, UC_DIRP1, MF_UNCHECKED);
		::CheckMenuItem(s_hSysMenu, UC_DIRP2, MF_CHECKED);
		s_trDirLeft = FALSE;
		ScoreLineDialog_Reflesh(hDlg, s_tabIndex);
	}
}

static void SysMenu_OnUndo(HWND hDlg, int x, int y)
{
	Memento_Undo();
	ScoreLineDialog_Reflesh(hDlg, s_tabIndex);
}

static void SysMenu_OnRedo(HWND hDlg, int x, int y)
{
	Memento_Redo();
	ScoreLineDialog_Reflesh(hDlg, s_tabIndex);
}

static void SysMenu_OnRankProfile(HWND hDlg, int x, int y)
{
	HWND hRankDlg = ::CreateDialog(g_hInstance, MAKEINTRESOURCE(IDD_SCORELINE_QIB), hDlg, RankProfDialog_DlgProc);
	::ShowWindow(hRankDlg, SW_SHOW);
}

static void SysMenu_OnTrackRecord(HWND hDlg, int x, int y)
{
	TrackRecordDialog_ShowModeless(hDlg, NULL);
}

static void ScoreLineDialog_InitSysMenu(HWND hDlg)
{
	int itemIndex = 0;
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION,				UC_OPENHIT,		"vt@Cǂݍ");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_SEPARATOR | MF_BYPOSITION, 0, NULL);
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION,				UC_UNDOSCR,		"ɖ߂");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION,				UC_REDOSCR,		"蒼");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_SEPARATOR | MF_BYPOSITION, 0, NULL);
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION,				UC_DIRP1,		"ʐ");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION,				UC_DIRP2,		"ʐ");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_SEPARATOR | MF_BYPOSITION, 0, NULL);
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION | MF_CHECKED,	UC_VIEWALL,		"SĂ̑ΐ");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION,				UC_VIEWL30,		"ߋRȎΐ");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION,				UC_VIEWL50,		"ߋTȎΐ");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION,				UC_VIEWL100,	"ߋPOȎΐ");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION,				UC_VIEWTOD,		"̑ΐ");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION,				UC_VIEWYES,		"̑ΐ");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION,				UC_VIEWCST,		"JX^ΐ");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_SEPARATOR | MF_BYPOSITION, 0, NULL);
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION,				UC_RNKPROF,		"vt@C\");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION,				UC_TRKRECD,		"ߋΐ헚\");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_SEPARATOR | MF_BYPOSITION, 0, NULL);
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION,				UC_OUTHTML,		"\HTMLɂďo");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_SEPARATOR | MF_BYPOSITION, 0, NULL);
}

static void ScoreLineDialog_InitTab(HWND hDlg)
{
	HWND hTabCtrl = ::GetDlgItem(hDlg, IDC_TAB);
	TCITEM tcItem;
	::ZeroMemory(&tcItem, sizeof tcItem);
	for(int i = 0; i < ARRAYSIZE(g_characters) + 1; ++i)
		TabCtrl_InsertItem(hTabCtrl, 0, &tcItem);

	tcItem.mask = TCIF_TEXT | TCIF_PARAM;
	for(int i = 0; i < ARRAYSIZE(g_characters) + 1; ++i) {
		int tabId = g_settings.ReadInteger(
			"DefaultDlg.Tab.Arrangement", Formatter("%d", i),
			i);
		if(tabId < 1) {	// VXe
			tcItem.pszText = "S";
			tcItem.lParam = 0;
		} else {	// LN^
			if(tabId > ARRAYSIZE(g_characters))
				tabId = ARRAYSIZE(g_characters);
			tcItem.pszText = (LPSTR)g_characters[tabId - 1].abbr;
			tcItem.lParam = tabId;
		}
		TabCtrl_SetItem(hTabCtrl, i, &tcItem);
	}
	TabCtrl_SetCurSel(hTabCtrl, 0);

	RECT rct;
	::GetClientRect(hTabCtrl, &rct);
	TabCtrl_AdjustRect(hTabCtrl, 0, &rct);
	TabCtrl_SetItemSize(hTabCtrl, (rct.right - rct.left) / 8 * 9 / 10, 20);

	s_origTabProc = (WNDPROC)GetWindowLong(hTabCtrl, GWL_WNDPROC);
	::SetWindowLong(hTabCtrl, GWL_WNDPROC, (LONG)TabCtrl_WindowProc);
}

static void ScoreLineDialog_InitEdit(HWND hDlg)
{
	for(int i = IDC_EDIT110; i <= IDC_EDIT431; ++i)
	{
		HWND hEditCtrl = GetDlgItem(hDlg, i);
		WNDPROC orgProc =
			(WNDPROC)::GetWindowLong(hEditCtrl, GWL_WNDPROC);

		::SetProp(hEditCtrl, PROP_OPROC, (HANDLE)orgProc);
		::SetWindowLong(hEditCtrl, GWL_WNDPROC, (LONG)EditCtrl_WindowProc);
	}

}

static BOOL ScoreLineDialog_OnInitDialog(HWND hDlg, HWND hwndFocus, LPARAM lParam)
{
	for(int i = 0; i < SWRCHAR_MAX; ++i) {
		s_label_specific[i] = "vs ";
		s_label_specific[i] += g_characters[s_convTable[i]].full;
		s_label_overall[i] = "with ";
		s_label_overall[i] += g_characters[s_convTable[i]].full;
	}
	s_label_specific[SWRCHAR_MAX] = "v";
	s_label_overall[SWRCHAR_MAX]  = "v";

	s_hSysMenu = ::GetSystemMenu(hDlg, FALSE);
	if(s_hSysMenu == NULL) return FALSE;
	
	ScoreLineDialog_InitSysMenu(hDlg);
	ScoreLineDialog_InitTab(hDlg);
	ScoreLineDialog_InitEdit(hDlg);

	HWND hTabCtrl = ::GetDlgItem(hDlg, IDC_TAB);
	TCITEM tcItem;
	tcItem.mask = TCIF_PARAM;
	TabCtrl_GetItem(hTabCtrl, 
		TabCtrl_GetCurSel(hTabCtrl), &tcItem);

	s_trDirLeft = g_settings.ReadInteger(
		"DefaultDlg.Trackrecord", "DirLeft", 1) != 0;
	::CheckMenuItem(s_hSysMenu, UC_DIRP1,  s_trDirLeft ? MF_CHECKED: MF_UNCHECKED);
	::CheckMenuItem(s_hSysMenu, UC_DIRP2, !s_trDirLeft ? MF_CHECKED: MF_UNCHECKED);

	int x, y;
	x = g_settings.ReadInteger("DefaultDlg.Display", "X", CW_USEDEFAULT);
	y = g_settings.ReadInteger("DefaultDlg.Display", "Y", CW_USEDEFAULT);
	if(x != CW_USEDEFAULT && y != CW_USEDEFAULT)
		::SetWindowPos(hDlg, 0, x, y, 0, 0, SWP_NOSIZE | SWP_NOZORDER);

	s_viewMode = STD_VIEW_ALL;
	return ScoreLineDialog_Reflesh(hDlg, tcItem.lParam);
}

static void ScoreLineDialog_OnSysCommand(HWND hDlg, UINT nID, int x, int y)
{
	if(nID == SC_CLOSE) {
		SysMenu_OnClose(hDlg, x, y);
	} else if(nID >= UC_VIEWALL && nID <= UC_VIEWYES) {
		SysMenu_OnView(hDlg, nID, x, y);
	} else if(nID == UC_VIEWCST) {
		SysMenu_OnViewCustom(hDlg, nID, x, y);
	} else if(nID == UC_OUTHTML) {
		SysMenu_OnOutputToHTML(hDlg, x, y);
	} else if(nID == UC_OPENHIT) {
		SysMenu_OnOpenProfile(hDlg, x, y);
	} else if(nID == UC_DIRP1) {
		SysMenu_OnDirectionP1(hDlg, x, y);
	} else if(nID == UC_DIRP2) {
		SysMenu_OnDirectionP2(hDlg, x, y);
	} else if(nID == UC_UNDOSCR) {
		SysMenu_OnUndo(hDlg, x, y);
	} else if(nID == UC_REDOSCR) {
		SysMenu_OnRedo(hDlg, x, y);
	} else if(nID == UC_RNKPROF) {
		SysMenu_OnRankProfile(hDlg, x, y);
	} else if(nID == UC_TRKRECD) {
		SysMenu_OnTrackRecord(hDlg, x, y);
	}
}

static void ScoreLineDialog_OnShortcut(HWND hDlg, int id)
{
	SHORTCUT sc;
	Shortcut_GetElement(sc, id - ID_SHORTCUT_BASE);
	ScoreLineDialog_OpenProfile(hDlg, sc.path);
}

static void ScoreLineDialog_OnCommand(HWND hDlg, int id, HWND hwndCtl, UINT codeNotify)
{
	if(id >= ID_SHORTCUT_BASE && id < ID_SHORTCUT_BASE + MAX_SHORTCUT) {
		ScoreLineDialog_OnShortcut(hDlg, id);
	}
}

static void ScoreLineDialog_OnUpdateScoreLine(HWND hDlg)
{
	ScoreLineDialog_Reflesh(hDlg, s_tabIndex);
}

static LRESULT ScoreLineDialog_OnNotify(HWND hDlg, int idCtrl, LPNMHDR pNMHdr)
{
	TCITEM tcItem;
	if(pNMHdr->idFrom == IDC_TAB) {
		switch(pNMHdr->code) {
		case TCN_SELCHANGE:
			tcItem.mask = TCIF_PARAM;
			TabCtrl_GetItem(pNMHdr->hwndFrom, 
				TabCtrl_GetCurSel(pNMHdr->hwndFrom), &tcItem);
			s_tabIndex = tcItem.lParam;
			ScoreLineDialog_Reflesh(hDlg, s_tabIndex);
			return TRUE;
		}
	}
	return FALSE;
}

static void ScoreLineDialog_OnDestroy(HWND hDlg)
{
	HWND hTabCtrl = ::GetDlgItem(hDlg, IDC_TAB);
	if(s_origTabProc)
		SetWindowLong(hTabCtrl, GWL_WNDPROC, (LONG)s_origTabProc);

	for(int i = IDC_EDIT110; i <= IDC_EDIT431; ++i)
	{
		HWND hEditCtrl = GetDlgItem(hDlg, i);
		WNDPROC orgProc = (WNDPROC)
			::GetProp(hEditCtrl, PROP_OPROC);
		if(orgProc) {
			::SetWindowLong(hEditCtrl, GWL_WNDPROC, (LONG)orgProc);
			::RemoveProp(hEditCtrl, PROP_OPROC);
		}
	}

	int count = TabCtrl_GetItemCount(hTabCtrl);
	TCITEM tcItem;
	tcItem.mask = TCIF_PARAM;
	for(int i = 0; i < count; ++i) {
		TabCtrl_GetItem(hTabCtrl, i, &tcItem);
		g_settings.WriteInteger(
			"DefaultDlg.Tab.Arrangement", Formatter("%d", i),
			tcItem.lParam);
	}
	RECT rct;
	::GetWindowRect(hDlg, &rct);
	g_settings.WriteInteger("DefaultDlg.Display", "X", rct.left);
	g_settings.WriteInteger("DefaultDlg.Display", "Y", rct.top);

	/* ܂Ȃ */
	s_tabIndex = 0;
}

static void ScoreLineDialog_OnNCRButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT codeHitTest)
{
	if(codeHitTest == HTCAPTION) {
		::EnableMenuItem(s_hSysMenu, UC_UNDOSCR, Memento_Undoable() ? MF_ENABLED : MF_GRAYED);
		::EnableMenuItem(s_hSysMenu, UC_REDOSCR, Memento_Redoable() ? MF_ENABLED : MF_GRAYED);
	}
}

BOOL CALLBACK ScoreLineDialog_DlgProc(HWND hDlg, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch(Msg) {
	HANDLE_DLG_MSG(hDlg, WM_INITDIALOG, ScoreLineDialog_OnInitDialog);
	HANDLE_DLG_MSG(hDlg, WM_DESTROY, ScoreLineDialog_OnDestroy);
	HANDLE_DLG_MSG(hDlg, WM_SYSCOMMAND, ScoreLineDialog_OnSysCommand);
	HANDLE_DLG_MSG(hDlg, WM_COMMAND, ScoreLineDialog_OnCommand);
	HANDLE_DLG_MSG(hDlg, WM_NCRBUTTONDOWN, ScoreLineDialog_OnNCRButtonDown);
	HANDLE_DLG_MSG(hDlg, WM_NOTIFY, ScoreLineDialog_OnNotify);
	case UM_UPDATESCORELINE: ScoreLineDialog_OnUpdateScoreLine(hDlg); break;
	}
	return FALSE;
}
